
# q-expansion of newform 304.3.q.b, downloaded from the LMFDB on 03 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 304
weight = 3
poly_data = [4699, -28298, 76145, -111558, 97124, -54402, 26325, -7446, 2400, -370, 85, -6, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[-2230631, 7158731, -9735843, 5434428, -3429485, 887825, -351615, 48734, -13451, 845, -169, 0], 2200], [[1472411636, -4991568921, 7296011113, -4690374933, 2894436180, -968590715, 333050615, -71001389, 14348986, -2030785, 205479, -18355], 2721400], [[634302201, -2146296897, 3124308244, -2017297821, 1240776773, -417225170, 143018825, -30674859, 6176600, -881601, 88719, -8034], 680350], [[646799427, -2405777591, 4083192375, -3356508268, 1987979837, -897563145, 266530115, -80302498, 13003143, -2790389, 207673, -32136], 2721400], [[-536286980, 1613073514, -1990691704, 841469792, -563573277, 50169095, -42104410, -5004635, -908936, -436281, -345, -8034], 680350], [[19270743, -129778210, 300464226, -340504370, 193368086, -110982770, 29732290, -11009562, 1579191, -412507, 26802, -5098], 340175], [[-67181747, 350782696, -726055950, 751893788, -431750602, 233914960, -64227030, 22701948, -3347643, 837384, -56078, 10196], 340175], [[4718721694, -18398151561, 31106841029, -25898476143, 15280653946, -6961616715, 2056651215, -624602171, 100528032, -21745637, 1607011, -250913], 2721400], [[1360159787, -1024149960, -4697250996, 11165702235, -5987054391, 4559690520, -1105862090, 493180817, -64267851, 19475742, -1153032, 250913], 2721400], [[-7322600864, 36268817795, -72832599663, 73414552335, -42279658448, 22484359645, -6219042925, 2165265711, -321926038, 79427931, -5366301, 962309], 2721400]]

hecke_ring_character_values = [[191, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [229, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [97, [-1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [1, -1, 0, 0, 0, 0, 0, -1, -1, 0, 0, 0], [0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0], [-2, 1, 0, 0, -1, 0, -1, 0, 3, 0, 0, 0], [1, 0, 0, 0, 0, 0, 0, 0, -2, -1, 1, 0], [-1, 0, 0, 0, 0, -1, -1, 0, 1, 0, 0, 1], [0, 0, 0, 1, -1, 0, -1, 0, 5, 1, -2, 1], [0, 0, -1, 1, -1, 2, 0, -1, 3, 0, -1, -1], [0, 0, 1, -1, 0, 1, -1, 3, 0, 0, -1, 1], [-6, 6, 3, 3, -1, -1, -3, 3, 6, -2, 1, 1], [3, 0, -2, -4, 1, -1, 0, 0, -6, 1, -1, 2], [-1, -2, -1, 0, 3, -1, -2, -4, -2, 0, 0, 0], [1, -1, 0, 5, 1, 0, 0, 1, -6, 0, 0, 1], [-22, 8, -6, -3, -1, 2, -4, 8, 15, 1, 0, -1], [-6, 0, -1, 1, 5, 0, 0, 4, -6, 0, 0, 0], [-9, 2, -8, -8, 1, -2, 0, 1, 9, -2, 1, 2], [2, -2, 10, 5, -3, 6, -4, -2, -2, 0, 0, -3], [0, -12, 3, 3, -2, 2, -2, -6, 0, 4, -2, -2], [-8, 0, 7, -7, 0, 0, 0, -7, -8, 0, 1, 0], [8, -6, 2, 1, 3, -6, 4, -6, -7, -1, 0, 3], [0, 0, 0, 1, -8, 0, -2, 0, -7, -1, 2, -4], [10, 12, 22, 11, 1, -2, -4, 12, 1, 1, 0, 1], [-8, -3, -9, -18, -2, 3, 1, 0, 19, -1, 1, -6], [-23, -4, -3, -3, -6, 3, -9, -2, 23, -2, 1, -3], [-17, 17, 0, -4, 1, 0, 4, -17, -2, -3, 6, -7], [8, 26, -13, -13, 1, 3, 5, 13, -8, -2, 1, -3], [-16, 10, 0, 0, 4, -2, 2, 0, 22, 8, -8, 4], [22, 12, -8, -16, 2, 2, 4, 0, -56, -2, 2, -4], [4, -4, 0, 4, 7, 0, 7, 4, -21, -4, 8, -7], [30, -14, 6, 0, -1, 9, -8, -28, -14, 0, 0, 0], [-15, 0, 3, -3, 9, -1, 1, 24, -15, 0, -9, -1], [-61, 1, 0, 0, -4, 8, 10, 1, 31, 4, 0, -4], [49, -42, 6, 6, -1, -1, -3, -21, -49, -10, 5, 1], [38, 0, 5, -5, 7, 5, -5, -4, 38, 0, 0, 5], [15, -15, 0, 5, -1, 0, -6, 15, -85, 3, -6, 11], [-56, -15, 8, 16, -13, -4, -17, 0, 127, 6, -6, 8], [17, -17, 0, -4, -14, 0, -5, 17, 60, 5, -10, -4], [-31, 8, 0, 0, 2, 2, 4, 0, 54, -13, 13, -4], [-63, 0, -9, 9, -8, 2, -2, -41, -63, 0, -6, 2], [-23, 23, 0, 16, -4, 0, -11, -23, -16, 1, -2, 18], [-17, 10, -1, -2, -20, 8, -12, 0, 24, -8, 8, -16], [-18, 16, 7, 7, 12, -8, 16, 8, 18, 12, -6, 8], [52, -36, -2, -4, -16, 0, -16, 0, -68, -6, 6, 0], [3, -3, 0, 19, -4, 0, -5, 3, -24, 2, -4, 6], [41, 4, 1, 0, 11, 9, -20, 8, 4, -4, -4, 0], [7, 0, -9, 9, -10, 0, 0, -31, 7, 0, -10, 0], [103, -45, -20, -10, 6, -12, -2, -45, -74, 17, 0, 6], [-12, -12, 14, 7, -8, 16, 11, -12, 0, -2, 0, -8], [-17, -38, -7, -14, 22, 6, 28, 0, 72, -10, 10, -12], [-55, -1, 36, 0, 10, 3, -13, -2, -1, -7, -7, 0], [19, -19, 0, 19, -11, 0, -18, 19, -24, 4, -8, 25], [3, -33, -14, -28, 18, 3, 21, 0, 27, -1, 1, -6], [-61, -14, 17, 17, -1, 7, 5, -7, 61, 16, -8, -7], [28, 0, -4, 4, -21, 3, -3, 34, 28, 0, -13, 3], [-123, 44, -21, -21, -4, -6, -14, 22, 123, -10, 5, 6], [352, -20, 2, 1, -8, 16, -5, -20, -186, -2, 0, -8], [-26, 26, 0, -22, -39, 0, -21, -26, -71, 4, -8, 3], [-56, 16, 14, 7, 2, -4, 29, 16, 36, 6, 0, 2], [25, -2, 15, 0, -1, -5, 6, -4, -2, 10, 10, 0], [9, -40, -1, -1, 18, 4, 40, -20, -9, -6, 3, -4], [-101, -35, -16, -8, 10, -20, 20, -35, 33, -14, 0, 10], [-25, 14, -27, 0, 5, 17, -22, 28, 14, -10, -10, 0], [110, 26, -14, -7, 7, -14, -30, 26, -42, 24, 0, 7], [-41, -30, 32, 64, -13, -1, -14, 0, 112, -13, 13, 2], [-39, 6, -49, -49, -3, -5, -11, 3, 39, 4, -2, 5], [47, -64, -6, -6, 20, -1, 39, -32, -47, -20, 10, 1], [-19, -24, 39, 78, -12, -10, -22, 0, 62, 18, -18, 20], [25, -25, 0, -36, 29, 0, 22, 25, -20, -17, 34, -15], [-86, -46, 58, 29, 7, -14, -10, -46, 20, 2, 0, 7], [113, 7, 16, 0, -2, 7, -5, 14, 7, 13, 13, 0], [218, -36, 0, 0, -27, 17, 10, -72, -36, -6, -6, 0], [-224, -52, -46, -23, 6, -12, 5, -52, 86, -6, 0, 6], [188, 0, 37, -37, 1, -12, 12, 4, 188, 0, 4, -12], [-96, -18, -60, 0, -14, 8, 6, -36, -18, 2, 2, 0], [-178, 6, 9, 18, 30, -18, 12, 0, 350, 15, -15, 36], [293, 23, -38, -19, -21, 42, 27, 23, -135, -7, 0, -21], [-5, 2, 42, 42, 5, 16, 26, 1, 5, -22, 11, -16], [-34, 34, 0, -19, 2, 0, 8, -34, 192, 16, -32, -14], [-5, 5, 0, -18, -53, 0, -18, -5, 92, 9, -18, -17], [-225, -12, -69, -69, 4, 6, 14, -6, 225, 2, -1, -6], [54, 22, -20, -40, 40, 2, 42, 0, -130, -2, 2, -4], [-2, 2, 0, -59, 24, 0, 26, -2, -322, -10, 20, -28], [-127, 0, -27, 27, -21, -15, 15, -22, -127, 0, 3, -15], [-125, 6, 67, 67, -21, -8, -50, 3, 125, -28, 14, 8], [97, 151, -34, -17, -17, 34, -25, 151, 27, -11, 0, -17], [220, 0, -12, 12, -3, 1, -1, 8, 220, 0, 17, 1], [-382, -26, 21, 0, 19, -1, -18, -52, -26, 9, 9, 0], [-45, -1, -57, 0, 7, -26, 19, -2, -1, -10, -10, 0], [0, 0, 0, -8, 85, 0, 41, 0, 95, 0, 0, 3], [9, -73, -26, -52, 42, 19, 61, 0, 55, 13, -13, -38], [-3, -38, -24, -48, 48, 2, 50, 0, 44, 11, -11, -4], [-3, 0, 19, -19, -28, 14, -14, -121, -3, 0, 10, 14], [242, -11, 30, 60, -19, -2, -21, 0, -473, -10, 10, 4], [451, 123, 54, 27, 0, 0, -46, 123, -164, 4, 0, 0], [-82, -106, -8, -4, -1, 2, -20, -106, -12, 13, 0, -1], [-82, 0, -1, 1, -10, 21, -21, -43, -82, 0, -3, 21], [-201, -92, -10, -10, 34, -33, 35, -46, 201, 8, -4, 33], [-177, -7, 42, 0, 27, -18, -9, -14, -7, 15, 15, 0], [-105, 0, -9, 9, -40, 26, -26, -59, -105, 0, -14, 26], [-181, 4, 46, 46, 18, -33, 3, 2, 181, 32, -16, 33], [-239, 0, -3, 3, -11, -13, 13, 94, -239, 0, -1, -13], [199, -62, -24, -24, 5, -4, 6, -31, -199, 38, -19, 4], [93, 52, 5, 10, -70, 28, -42, 0, -238, -20, 20, -56], [191, 95, -11, 0, 12, -21, 9, 190, 95, -2, -2, 0], [11, 154, 11, 22, 20, -8, 12, 0, -176, -24, 24, 16], [-36, -24, 54, 0, -9, -17, 26, -48, -24, -6, -6, 0], [-289, -65, -10, -5, 0, 0, -2, -65, 112, 32, 0, 0], [-91, 86, -35, -35, 21, -43, -1, 43, 91, -24, 12, 43], [94, 0, 1, -1, -60, 7, -7, 3, 94, 0, -67, 7], [-114, -72, 25, 0, -21, -15, 36, -144, -72, -19, -19, 0], [-196, 61, 10, 20, 25, 2, 27, 0, 331, -48, 48, -4], [-70, 70, 0, -70, 23, 0, -7, -70, 317, 4, -8, 37], [-25, 34, 0, 0, 5, 49, 59, 17, 25, 18, -9, -49], [-224, 54, 8, 16, -18, -8, -26, 0, 394, -4, 4, 16], [-42, 0, 1, -1, 55, 2, -2, -14, -42, 0, -36, 2], [-28, 28, 0, 3, -42, 0, 16, -28, 369, -7, 14, -74], [282, 138, -28, -14, -27, 54, 14, 138, -72, 11, 0, -27], [-36, 10, 42, 84, -28, -22, -50, 0, 62, 54, -54, 44], [223, 49, 64, 0, 26, 37, -63, 98, 49, 7, 7, 0], [-227, 0, 15, -15, 42, -10, 10, 99, -227, 0, -10, -10], [-542, -34, -29, -29, -33, 43, -23, -17, 542, 6, -3, -43], [-67, -49, -17, 0, -46, 41, 5, -98, -49, 6, 6, 0], [-167, 11, 98, 0, 40, -3, -37, 22, 11, -1, -1, 0], [-195, 73, 6, 12, 22, 17, 39, 0, 317, 9, -9, -34], [-36, 8, -96, -192, 52, -16, 36, 0, 64, 20, -20, 32], [-75, 75, 0, -47, -9, 0, 48, -75, 145, -31, 62, -105], [-138, 12, 55, 55, -58, 42, -74, 6, 138, -32, 16, -42], [687, 7, 78, 39, 18, -36, -18, 7, -340, 0, 0, 18], [55, -21, -166, -83, 26, -52, 50, -21, -38, -20, 0, 26], [67, -64, -89, 0, -31, 43, -12, -128, -64, -20, -20, 0], [-244, -96, 34, 17, 5, -10, -18, -96, 74, -20, 0, 5], [50, 58, 2, 1, 44, -88, 73, 58, 4, -8, 0, 44], [-107, 0, -1, 1, 37, -57, 57, -36, -107, 0, -5, -57], [13, -13, 0, -64, -20, 0, 5, 13, -164, -3, 6, -30], [34, -12, 60, 0, 3, -37, 34, -24, -12, 26, 26, 0], [23, 166, -49, -49, -37, 28, -46, 83, -23, -28, 14, -28], [-206, 68, -7, -7, -18, 68, 32, 34, 206, -12, 6, -68], [-21, -82, 59, 118, -52, 36, -16, 0, 124, -24, 24, -72], [-45, 50, 19, 0, -17, -25, 42, 100, 50, 16, 16, 0], [234, 20, 59, 0, -35, 7, 28, 40, 20, 3, 3, 0], [-17, 0, 19, -19, -6, 16, -16, -85, -17, 0, -4, 16], [-77, 88, 164, 164, -24, 35, -13, 44, 77, 56, -28, -35], [-1, 0, 65, -65, 40, 6, -6, 67, -1, 0, 42, 6], [82, 0, 44, -44, -9, -39, 39, -158, 82, 0, 57, -39], [236, -86, -52, 0, 6, -36, 30, -172, -86, 46, 46, 0], [96, -108, 74, 37, 44, -88, -33, -108, -102, -2, 0, 44], [67, -67, 0, 18, 54, 0, -27, 67, -70, 25, -50, 108], [90, -90, 0, -8, 116, 0, 78, 90, 79, -30, 60, -40], [-22, 0, -22, 22, -66, 22, -22, 157, -22, 0, -18, 22], [185, -30, -62, -124, 17, -43, -26, 0, -340, -25, 25, 86], [19, -19, 0, 57, 37, 0, 8, 19, -797, -37, 74, 21], [-536, -56, 7, 0, 65, -65, 0, -112, -56, 13, 13, 0], [42, 0, -40, 40, -67, -5, 5, -104, 42, 0, 3, -5], [-27, 0, -75, 75, -46, 8, -8, -79, -27, 0, -34, 8], [-98, 62, 82, 41, -7, 14, 16, 62, 80, -2, 0, -7], [84, -27, 54, 108, 99, -28, 71, 0, -141, 56, -56, 56], [110, -104, -66, -132, -68, -8, -76, 0, -116, -24, 24, 16], [-36, 36, 0, -140, 74, 0, 22, -36, 269, 36, -72, 30], [29, 120, 47, 47, 10, -88, -68, 60, -29, 82, -41, 88], [355, 90, 86, 86, 21, 10, 52, 45, -355, 18, -9, -10], [260, 146, -30, -15, -13, 26, 58, 146, -57, 25, 0, -13], [-32, 32, 0, -33, -62, 0, -28, -32, 641, 13, -26, -6], [90, 8, 162, 81, -7, 14, 8, 8, -41, 53, 0, -7], [-486, 42, 162, 81, -19, 38, -48, 42, 264, -18, 0, -19], [-410, 96, 132, 0, 23, 11, -34, 192, 96, -26, -26, 0], [142, -56, -166, -83, 29, -58, 76, -56, -99, -1, 0, 29], [484, 74, -50, -25, -17, 34, 104, 74, -205, -89, 0, -17], [-122, 164, -33, -33, -6, -12, -24, 82, 122, 16, -8, 12]]
